#include "StdAfx.h"
#include "SegDllUtil.h"

#include <eh.h>
#include <TCHAR.h>
#include <objbase.h>

static void err_translator(unsigned code, EXCEPTION_POINTERS *)
{
	throw code;
}

CSegDllUtil::CSegDllUtil(void)
{
	m_hModule = NULL;
	CoInitialize(NULL);
}

CSegDllUtil::~CSegDllUtil(void)
{
	UnLoadDLL();
	CoUninitialize();
}

BOOL CSegDllUtil::LoadDLL()
{
	_set_se_translator(err_translator);

	if( m_hModule != NULL ){
		return FALSE;
	}

	BOOL bRet = TRUE;

	try{
		m_hModule = LoadLibrary(_T("SegDriver_hdus.dll"));
		if( m_hModule == NULL ){
//			AfxMessageBox( NULL, _T("SegDriver_hdus.dll ̃[hɎs܂"), _T(""), MB_OK);
			return FALSE;
		}
	}
	catch(...){
		return FALSE;
	}

	pfnGetSegments = ( GetSegmentsDLL ) ::GetProcAddress( m_hModule , "GetSegments");
	if( !pfnGetSegments ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnGetDLLVersion = ( GetDLLVersionDLL ) ::GetProcAddress( m_hModule , "GetDLLVersion");
	if( !pfnGetDLLVersion ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnGetDLLDescriptor = ( GetDLLDescriptorDLL ) ::GetProcAddress( m_hModule , "GetDLLDescriptor");
	if( !pfnGetDLLDescriptor ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnGetDLLName = ( GetDLLNameDLL ) ::GetProcAddress( m_hModule , "GetDLLName");
	if( !pfnGetDLLName ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnSetLogCallback = ( SetLogCallbackDLL ) ::GetProcAddress( m_hModule , "SetLogCallback");
	if( !pfnSetLogCallback ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnSetReceiveTSCallback = ( SetReceiveTSCallbackDLL ) ::GetProcAddress( m_hModule , "SetReceiveTSCallback");
	if( !pfnSetReceiveTSCallback ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnDeactivate = ( DeactivateDLL ) ::GetProcAddress( m_hModule , "Deactivate");
	if( !pfnDeactivate ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnTune = ( TuneDLL ) ::GetProcAddress( m_hModule , "Tune");
	if( !pfnTune ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnActivate = ( ActivateDLL ) ::GetProcAddress( m_hModule , "Activate");
	if( !pfnActivate ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnGetTunerName = ( GetTunerNameDLL ) ::GetProcAddress( m_hModule , "GetTunerName");
	if( !pfnGetTunerName ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnGetTunerID = ( GetTunerIDDLL ) ::GetProcAddress( m_hModule , "GetTunerID");
	if( !pfnGetTunerID ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnGetTunerCount = ( GetTunerCountDLL ) ::GetProcAddress( m_hModule , "GetTunerCount");
	if( !pfnGetTunerCount ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnFreeDriverInstance = ( FreeDriverInstanceDLL ) ::GetProcAddress( m_hModule , "FreeDriverInstance");
	if( !pfnFreeDriverInstance ){
		bRet = FALSE;
		goto ERR_END;
	}
	pfnMakeDriverInstance = ( MakeDriverInstanceDLL ) ::GetProcAddress( m_hModule , "MakeDriverInstance");
	if( !pfnMakeDriverInstance ){
		bRet = FALSE;
		goto ERR_END;
	}

ERR_END:
	if( bRet == FALSE ){
		::FreeLibrary( m_hModule );
		m_hModule=NULL;
//		MessageBox( NULL, _T("GetProcAddress Ɏs܂"), _T(""), MB_OK);
	}
	return bRet;
}

void CSegDllUtil::UnLoadDLL()
{
	if( m_hModule != NULL ){
		::FreeLibrary( m_hModule );
		m_hModule=NULL;
	}
}

DWORD CSegDllUtil::GetSegments()
{
	if( m_hModule == NULL ){
		return 0;
	}
	return pfnGetSegments();
}

DWORD CSegDllUtil::GetDLLVersion()
{
	if( m_hModule == NULL ){
		return 0;
	}
	return pfnGetDLLVersion();
}

LPCSTR CSegDllUtil::GetDLLDescriptor()
{
	if( m_hModule == NULL ){
		return NULL;
	}
	return pfnGetDLLDescriptor();
}

LPCSTR CSegDllUtil::GetDLLName()
{
	if( m_hModule == NULL ){
		return NULL;
	}
	return pfnGetDLLName();
}

void CSegDllUtil::SetLogCallback(void* pDrv,void* pLogMethod)
{
	if( m_hModule == NULL ){
		return ;
	}
	return pfnSetLogCallback(pDrv, (TLogMethod)pLogMethod);
}

void CSegDllUtil::SetReceiveTSCallback(void* pDrv,void* pTSMethod)
{
	if( m_hModule == NULL ){
		return ;
	}
	return pfnSetReceiveTSCallback(pDrv, (TSMethod)pTSMethod);
}

int CSegDllUtil::Deactivate(void* pDrv)
{
	if( m_hModule == NULL ){
		return -1;
	}
	return pfnDeactivate(pDrv);
}

int CSegDllUtil::Tune(void* pDrv, int iFreq)
{
	if( m_hModule == NULL ){
		return -1;
	}
	return pfnTune(pDrv, iFreq);
}

int CSegDllUtil::Activate(void* pDrv, LPCSTR lpszID)
{
	if( m_hModule == NULL ){
		return -1;
	}
	return pfnActivate(pDrv, lpszID);
}

void CSegDllUtil::GetTunerName(int iIndex, char* lpszName, int iLength)
{
	if( m_hModule == NULL ){
		return ;
	}
	return pfnGetTunerName(iIndex, lpszName, iLength);
}

void CSegDllUtil::GetTunerID(int iIndex, char* lpszID, int iLength)
{
	if( m_hModule == NULL ){
		return ;
	}
	return pfnGetTunerID(iIndex, lpszID, iLength);
}

int CSegDllUtil::GetTunerCount(void)
{
	if( m_hModule == NULL ){
		return -1;
	}
	return pfnGetTunerCount();
}

void CSegDllUtil::FreeDriverInstance(void* pDrv)
{
	if( m_hModule == NULL ){
		return ;
	}
	return pfnFreeDriverInstance(pDrv);
}

void* CSegDllUtil::MakeDriverInstance(void)
{
	if( m_hModule == NULL ){
		return NULL;
	}
	return pfnMakeDriverInstance();
}
